@@ -2,7 +2,7 @@ import codecs
 import json
 import os
 from datetime import datetime
-from typing import List
+from typing import List, Union
 from urllib.parse import urljoin
 import requests
 from bs4 import BeautifulSoup
@@ -24,19 +24,26 @@ class JiraService:
         :param config: a dict like this, if something is missing, it will get from env variable
                 {
                     "server": "https://jira.teko.vn",
-                    "username": "username",
-                    "password": "password",
+                    "jira_username": "jira_username",
+                    "jira_password": "jira_password",
+                    "confluence_username": "confluence_username",
+                    "confluence_password": "confluence_password",
                     "project_key": "PRJ",
                 }
         """
+
+        self.success = None
+
         load_dotenv()
         self.project_key = config.get('project_key') or os.getenv('JIRA_PROJECT_KEY')
         server = config.get('server') or os.getenv('JIRA_SERVER')
-        username = config.get('username') or os.getenv('JIRA_USERNAME')
-        password = config.get('password') or os.getenv('JIRA_PASSWORD')
+        jira_username = config.get('jira_username') or os.getenv('JIRA_USERNAME')
+        jira_password = config.get('jira_password') or os.getenv('JIRA_PASSWORD')
+        confluence_username = config.get('confluence_username') or os.getenv('CONFLUENCE_USERNAME')
+        confluence_password = config.get('confluence_password') or os.getenv('CONFLUENCE_PASSWOPRD')
 
-        if not self.project_key or not server or not username or not password:
-            msg = "No valide JIRA configuration found, please set JIRA_SERVER, JIRA_PROJECT_KEY, " \
+        if not self.project_key or not server or not jira_username or not jira_password:
+            msg = "[TEKO TOOL] No valid JIRA configuration found, please set JIRA_SERVER, JIRA_PROJECT_KEY, " \
                   "JIRA_USERNAME and JIRA_PASSWORD environment variables first"
             CLog.error(msg)
             raise TekoJiraException(msg)
@@ -45,10 +52,12 @@ class JiraService:
             server = "https://" + server
         self.base_api_url = urljoin(server, '/rest/atm/1.0')
         self.base_api_tests_url = urljoin(server, '/rest/tests/1.0')
-        # self.issue_url = jira_settings['url'] + '/rest/api/latest/issue'
 
-        self.s = requests.session()
-        self.s.auth = (username, password)
+        self.jira_session = requests.session()
+        self.jira_session.auth = (jira_username, jira_password)
+
+        self.confluence_session = requests.session()
+        self.confluence_session.auth = (confluence_username, confluence_password)
 
     def search_testcases(self, issues=[], name=None):
         """
@@ -68,17 +77,20 @@ class JiraService:
 
         url = self.base_api_url + "/testcase/search"
 
-        res = self.s.get(url=url, params=params)
+        res = self.jira_session.get(url=url, params=params)
 
         if res.status_code != 200:
-            CLog.error(f"HTTP Error {res.status_code}: {res.text}")
+            CLog.error(f"[TEKO TOOL] Failed to search for testcase {name}")
+            if res.status_code == 403:
+                CLog.error(f"[TEKO TOOL] Forbidden: You don't have permission to send request GET to {url} with params {params}")
+            else:
+                CLog.error(f"[TEKO TOOL] HTTP Error {res.status_code}: {res.text}")
             return []
         raw = res.text
 
         testcases_json = json.loads(raw)
-        # print(*testcases_json, sep="\n")
 
-        print(json.dumps(testcases_json))
+        print(f"[TEKO TOOL] {json.dumps(testcases_json)}")
 
         testcases = []
         for t in testcases_json:
@@ -86,26 +98,34 @@ class JiraService:
         return testcases
 
     def delete_trace_link(self, link_id):
-        url = self.base_api_url + "/tracelink/{link_id}"
+        url = self.base_api_url + f"/tracelink/{link_id}"
 
-        res = self.s.delete(url=url)
+        res = self.jira_session.delete(url=url)
 
         if res.status_code != 200:
-            CLog.error(f"HTTP Error {res.status_code}: {res.text}")
-            return []
+            CLog.error(f"[TEKO TOOL] Failed to delete trace link {link_id}")
+            if res.status_code == 403:
+                CLog.error(f"[TEKO TOOL] Forbidden: You don't have permission to send request DELETE to {url}")
+            else:
+                CLog.error(f"[TEKO TOOL] HTTP Error {res.status_code}: {res.text}")
+            return
         raw = res.text
 
         testcases_json = json.loads(raw)
 
-        print(json.dumps(testcases_json))
+        print(f"[TEKO TOOL] {json.dumps(testcases_json)}")
 
     def empty_trace_links(self, test_id):
         url_confluence = self.base_api_tests_url + f"/testcase/{test_id}/tracelinks/confluencepage"
 
-        res = self.s.get(url=url_confluence)
+        res = self.jira_session.get(url=url_confluence)
 
         if res.status_code != 200:
-            CLog.error(f"HTTP Error {res.status_code}: {res.text}")
+            CLog.error(f"[TEKO TOOL] Failed to clear confluence links for testcase {test_id}")
+            if res.status_code == 403:
+                CLog.error(f"[TEKO TOOL] Forbidden: You don't have permission to send request GET to {url_confluence}")
+            else:
+                CLog.error(f"[TEKO TOOL] HTTP Error {res.status_code}: {res.text}")
         else:
             raw = res.text
             links = json.loads(raw)
@@ -114,25 +134,40 @@ class JiraService:
 
         url_web = self.base_api_tests_url + f"/testcase/{test_id}/tracelinks/weblink"
 
-        res = self.s.get(url=url_web)
+        res = self.jira_session.get(url=url_web)
 
         if res.status_code != 200:
-            CLog.error(f"HTTP Error {res.status_code}: {res.text}")
+            CLog.error(f"[TEKO TOOL] Failed to clear web links for testcase {test_id}")
+            if res.status_code == 403:
+                CLog.error(f"[TEKO TOOL] Forbidden: You don't have permission to send request GET to {url_web}")
+            else:
+                CLog.error(f"[TEKO TOOL] HTTP Error {res.status_code}: {res.text}")
         else:
             raw = res.text
             links = json.loads(raw)
             for link in links:
                 self.delete_trace_link(link["id"])
 
-    def add_links(self, test_id, confluences: List[str], webs: List[dict]):
-        self.empty_trace_links(test_id)
-
+    def add_confluence_links(self, test_id, confluences: List[str]):
         url = self.base_api_tests_url + "/tracelink/bulk/create"
 
+        if not confluences:
+            return
+
         confluences_data = []
 
         for confluence_url in confluences:
-            html = self.s.get(url=confluence_url)
+            res = self.confluence_session.get(url=confluence_url)
+            if res.status_code != 200:
+                CLog.error(f"[TEKO TOOL] Failed to get id of confluence page {confluence_url}")
+                if res.status_code == 403:
+                    CLog.error(f"[TEKO TOOL] Forbidden: You don't have permission to send request GET to {confluence_url}")
+                else:
+                    CLog.error(f"[TEKO TOOL] HTTP Error {res.status_code}: {res.text}")
+                self.success = False
+                return
+
+            html = res.text
             soup = BeautifulSoup(html.text, "html.parser")
             page_id = soup.find("meta", {"name": "ajs-page-id"})["content"]
 
@@ -142,6 +177,22 @@ class JiraService:
                 "typeId": 1
             })
 
+        res = self.jira_session.post(url=url, json=confluences_data)
+
+        if res.status_code != 200:
+            CLog.error(f"[TEKO TOOL] Failed to add confluence links for testcase {test_id}")
+            if res.status_code == 403:
+                CLog.error(f"[TEKO TOOL] Forbidden: You don't have permission to send request POST to {url} with payload {confluences_data}")
+            else:
+                CLog.error(f"[TEKO TOOL] HTTP Error {res.status_code}: {res.text}")
+            self.success = False
+
+    def add_web_links(self, test_id, webs: List[Union[str, dict]]):
+        url = self.base_api_tests_url + "/tracelink/bulk/create"
+
+        if not webs:
+            return
+
         webs_data = []
 
         for web in webs:
@@ -160,17 +211,22 @@ class JiraService:
                     "typeId": 1
                 })
 
-        if confluences_data:
-            res = self.s.post(url=url, json=confluences_data)
+        res = self.jira_session.post(url=url, json=webs_data)
 
-            if res.status_code != 200:
-                CLog.error(f"HTTP Error {res.status_code}: {res.text}")
+        if res.status_code != 200:
+            CLog.error(f"[TEKO TOOL] Failed to add web links for testcase {test_id}")
+            if res.status_code == 403:
+                CLog.error(f"[TEKO TOOL] Forbidden: You don't have permission to send request POST to {url} with payload {webs_data}")
+            else:
+                CLog.error(f"[TEKO TOOL] HTTP Error {res.status_code}: {res.text}")
+            self.success = False
 
-        if webs_data:
-            res = self.s.post(url=url, json=webs_data)
+    def add_links(self, test_id, confluences: List[str], webs: List[dict]):
+        self.empty_trace_links(test_id)
 
-            if res.status_code != 200:
-                CLog.error(f"HTTP Error {res.status_code}: {res.text}")
+        self.add_confluence_links(test_id, confluences)
+
+        self.add_web_links(test_id, webs)
 
     def create_folder(self, name: str, type: str):
         data = {
@@ -184,30 +240,40 @@ class JiraService:
         """
         Ignore failure
         """
-        res = self.s.post(url=url, json=data)
+        res = self.jira_session.post(url=url, json=data)
+        if res.status_code == 403:
+            CLog.error(f"[TEKO TOOL] Cannot create {type} folder {name}")
+            CLog.error(f"[TEKO TOOL] Forbidden: You don't have permission to send request POST to {url} with payload {data}")
         if res.status_code != 201:
-            CLog.info(f"Folder {name} has already exists")
+            CLog.info(f"[TEKO TOOL] {type} Folder {name} has already exists")
 
     def get_test_id(self, test_key):
         url = self.base_api_tests_url + f"/testcase/{test_key}"
 
-        res = self.s.get(url=url, params={"fields": "id"})
+        res = self.jira_session.get(url=url, params={"fields": "id"})
         if res.status_code != 200:
-            CLog.error(f"HTTP Error {res.status_code}: {res.text}")
+            CLog.error(f"[TEKO TOOL] Failed to get id of testcase {test_key}")
+            if res.status_code == 403:
+                CLog.error(f"[TEKO TOOL] Forbidden: You don't have permission to send request GET to {url}")
+            else:
+                CLog.error(f"[TEKO TOOL] HTTP Error {res.status_code}: {res.text}")
+            self.success = False
             return
+
         raw = res.text
         return json.loads(raw).get('id')
 
     def create_or_update_testcases_by_name(self, testcases: List[JiraTestcase], issues=[]):
+        self.success = True
         for testcase in testcases:
             existing_testcases = self.search_testcases(issues=issues, name=testcase.name)
             if not existing_testcases:
                 key = self.create_testcase(testcase)
-                CLog.info(f"New testcase created: {key}")
+                CLog.info(f"[TEKO TOOL] New testcase created: {key}")
                 testcase.key = key
             else:
                 key = self.update_testcase(existing_testcases[0].key, testcase)
-                CLog.info(f"Existing testcase updated: {key}")
+                CLog.info(f"[TEKO TOOL] Existing testcase updated: {key}")
                 testcase.key = key
             if key:
                 self.add_links(
@@ -215,6 +281,10 @@ class JiraService:
                     confluences=testcase.confluenceLinks,
                     webs=testcase.webLinks
                 )
+        if not self.success:
+            msg = "[TEKO TOOL] Creating/updating test cases process do not succeeded"
+            CLog.error(msg)
+            raise TekoJiraException(msg)
         return testcases
 
     def prepare_testcase_data(self, testcase: JiraTestcase):
@@ -230,6 +300,8 @@ class JiraService:
             'status': testcase.status
         }
 
+        data.pop("status")
+
         if testcase.testScript:
             testscript = testcase.testScript.to_dict()
             testscript.pop("id")
@@ -249,14 +321,19 @@ class JiraService:
 
         return data
 
-    def update_testcase(self, key:str, testcase: JiraTestcase):
+    def update_testcase(self, key: str, testcase: JiraTestcase):
         url = self.base_api_url + f"/testcase/{key}"
         data = self.prepare_testcase_data(testcase)
         data.pop('projectKey')
-        res = self.s.put(url=url, json=data)
+        res = self.jira_session.put(url=url, json=data)
 
         if res.status_code != 200:
-            CLog.error(f"HTTP Error {res.status_code}: {res.text}")
+            CLog.error(f"[TEKO TOOL] Failed to update testcase `{testcase.name}`")
+            if res.status_code == 403:
+                CLog.error(f"[TEKO TOOL] Forbidden: You don't have permission to send request PUT to {url} with payload {data}")
+            else:
+                CLog.error(f"HTTP Error {res.status_code}: {res.text}")
+            self.success = False
             return None
 
         return key
@@ -264,18 +341,21 @@ class JiraService:
     def create_testcase(self, testcase: JiraTestcase):
         data = self.prepare_testcase_data(testcase)
         url = self.base_api_url + "/testcase"
-        res = self.s.post(url=url, json=data)
+        res = self.jira_session.post(url=url, json=data)
 
-        # print("data dict:", data)
-        # print("url:", url)
-
-        if res.status_code != 201:
-            CLog.error(f"HTTP Error {res.status_code}: {res.text}")
+        if res.status_code != 200:
+            CLog.error(f"[TEKO TOOL] Failed to create testcase `{testcase.name}`")
+            if res.status_code == 403:
+                CLog.error(f"[TEKO TOOL] Forbidden: You don't have permission to send request POST to {url} with payload {data}")
+            else:
+                CLog.error(f"HTTP Error {res.status_code}: {res.text}")
+            self.success = False
             return None
+
         raw = res.text
 
         testcase_json = json.loads(raw)
-        print(json.dumps(testcase_json))
+        print(f"[TEKO TOOL] {json.dumps(testcase_json)}")
 
         return testcase_json['key']
 
@@ -294,18 +374,18 @@ class JiraService:
         return data
 
     def create_test_cycle(self, testcases: List[JiraTestcase]):
+        self.success = True
         items_by_folder = dict()
         for testcase in testcases:
             existing_testcases = self.search_testcases(name=testcase.name)
             if not existing_testcases:
-                CLog.warn(f"Testcase with name `{testcase.name}` not existed!")
+                CLog.warn(f"[TEKO TOOL] Testcase with name `{testcase.name}` not existed!")
             else:
                 testcase.key = existing_testcases[0].key
-                CLog.info(f"Testcase with name `{testcase.name}` existed: #{testcase.key}")
+                CLog.info(f"[TEKO TOOL] Testcase with name `{testcase.name}` existed: #{testcase.key}")
                 if items_by_folder.get(testcase.testrun_folder) is None:
                     items_by_folder[testcase.testrun_folder] = []
                 items_by_folder[testcase.testrun_folder].append(self.prepare_testrun_data(testcase))
-        # print("items:", items)
 
         url = self.base_api_url + "/testrun"
 
@@ -322,20 +402,30 @@ class JiraService:
             else:
                 self.create_folder(folder, "TEST_RUN")
 
-            print("data dict: ", data)
+            print(f"[TEKO TOOL] Data: {data}")
 
-            res = self.s.post(url=url, json=data)
+            res = self.jira_session.post(url=url, json=data)
 
-            if res.status_code != 201:
-                CLog.error(f"HTTP Error {res.status_code}: {res.text}")
+            if res.status_code != 200:
+                CLog.error(f"[TEKO TOOL] Failed to create test cycle`")
+                if res.status_code == 403:
+                    CLog.error(f"[TEKO TOOL] Forbidden: You don't have permission to send request POST to {url} with payload {data}")
+                else:
+                    CLog.error(f"HTTP Error {res.status_code}: {res.text}")
+                self.success = False
                 return
+
             raw = res.text
 
             testcase_json = json.loads(raw)
-            print(json.dumps(testcase_json))
+            print(f"[TEKO TOOL] {json.dumps(testcase_json)}")
 
             key = testcase_json['key']
-            CLog.info(f"Test cycle `{key}` created!")
+            CLog.info(f"[TEKO TOOL] Test cycle `{key}` created!")
+        if not self.success:
+            msg = "[TEKO TOOL] Creating test cycle process does not succeeded"
+            CLog.error(msg)
+            raise TekoJiraException(msg)
 
     @staticmethod
     def read_testcases_from_file(testcase_file):
@@ -345,27 +435,42 @@ class JiraService:
         :return:
         """
         filename, ext = os.path.splitext(testcase_file)
-        print(os.path.abspath(testcase_file))
+        print(f"[TEKO TOOL] {os.path.abspath(testcase_file)}")
         if ext == ".json":
             with codecs.open(testcase_file, "r", encoding="utf-8") as f:
-                testcase_data = json.load(f)
+                try:
+                    testcase_data = json.load(f)
+                except Exception:
+                    msg = "[TEKO TOOL] Invalid json format"
+                    CLog.error(msg)
+                    raise TekoJiraException(msg) from None
+
         elif ext == ".yaml":
             with codecs.open(testcase_file, "r", encoding="utf-8") as f:
                 yaml = YAML(typ="safe")
-                testcase_data = yaml.load(f)
+                try:
+                    testcase_data = yaml.load(f)
+                except Exception:
+                    msg = "[TEKO TOOL] Invalid yaml format"
+                    CLog.error(msg)
+                    raise TekoJiraException(msg) from None
         else:
-            msg = f"File format not supported: {ext}"
+            msg = f"[TEKO TOOL] File format not supported: {ext}"
             CLog.error(msg)
             raise TekoJiraException(msg)
 
-        # print("data:", json.dumps(testcase_data))
         testcases = []
         for t in testcase_data:
             if t.get('folder') and t['folder'][0] != '/':
                 t['folder'] = '/' + t['folder']
             if t.get('testrun_folder') and t['testrun_folder'][0] != '/':
                 t['testrun_folder'] = '/' + t['testrun_folder']
-            testcases.append(JiraTestcase().from_dict(t))
+            try:
+                testcases.append(JiraTestcase().from_dict(t))
+            except Exception:
+                msg = f"[TEKO TOOL] Failed to parse test data from file {testcase_file}"
+                CLog.error(msg)
+                raise TekoJiraException(msg) from None
 
         return testcases
 
@@ -378,7 +483,7 @@ class JiraService:
         """
         testcases_json = []
         for t in testcases:
-            print(t)
+            print(f"[TEKO TOOL] {t}")
             data = t.to_dict()
             testcases_json.append(data)
 
@@ -393,55 +498,29 @@ class JiraService:
                 yaml.indent(offset=2)
                 yaml.dump(testcases_json, f)
         else:
-            msg = f"File format not supported: {ext}"
+            msg = f"[TEKO TOOL] File format not supported: {ext}"
             CLog.error(msg)
-            raise TekoJiraException(msg)
+            raise TekoJiraException(msg) from None
 
         return None
 
 
-def test():
-    jira_srv = JiraService()
-
-    # testscript = JiraTestScript(text="From ABC to XYZ")
-    testscript = JiraTestScript(type="STEP_BY_STEP",
-                                steps=[JiraTestScriptStep(description="<b>Step 1</b>",
-                                                          testData="<strong>XYZ</strong>",
-                                                          expectedResult="res1"),
-                                       JiraTestScriptStep(description="Step2", expectedResult="res2"),
-                                       JiraTestScriptStep(description="Step3", expectedResult="res3"),
-                                       ])
-    testcase = JiraTestcase(name="THUC TEST UNIQUE 3",
-                            objective="Test submit testcase from Teko cli tools 2",
-                            testScript=testscript)
-
-    # key = jira_srv.create_or_update_testcase_by_name(testcase, issues=['TESTING-1'])
-
-    testcases = jira_srv.search_testcases()
-    # testcases = jira_srv.search_testcases(issues=['TESTING-1'], name="THUC TEST UNIQUE 3")
-
-    for t in testcases:
-        print(t)
-
-
 if __name__ == "__main__":
-    # test()
-
     jira_srv = JiraService()
 
     testcase_file = "../../../sample/testcases.json"
     testcases = JiraService.read_testcases_from_file(testcase_file)
 
     for t in testcases:
-        print(t)
+        print(f"[TEKO TOOL] {t}")
 
     jira_srv.create_or_update_testcases_by_name(testcases)
 
     print("\nAfter:\n")
     for t in testcases:
-        print(t)
+        print(f"[TEKO TOOL] {t}")
 
-    testcycle_file = "../../../sample/testcycles.json"
+    testcycle_file = "../../../sample/testcases.json"
     testcases = JiraService.read_testcases_from_file(testcycle_file)
     jira_srv.create_test_cycle(testcases)
 
